package encapsulation.app;

import encapsulation.export.ResultsWriter;
import encapsulation.export.TextWriter;
import encapsulation.metrics.AccessMetricCalculator;
import encapsulation.metrics.FieldAccessStrategy;
import encapsulation.metrics.FieldMemberStrategy;
import encapsulation.metrics.MemberMetricCalculator;
import encapsulation.metrics.MethodAccessStrategy;
import encapsulation.metrics.MethodMemberStrategy;
import encapsulation.model.Program;

/** 
 * The main entry-point for the application.
 * Loads up the model of the program and uses a visitor to build a simplified model.
 * It then creates metrics calculators to calculate metrics and a results writer 
 * to write the measurements out to file.
 */
public class MetricsApp {

	/** 
	 * Run the application.
     * @param args An array of file or directory names containing parse trees.
     */
    public static void main(String[] args) {

    	//load the parsetrees
    	ParsetreeLoader loader = new ParsetreeLoader();
    	loader.loadParsetrees(args[0]);
		
    	//visit the JST model and build a simpler model of the program
    	JSTModelVisitor visitor = new JSTModelVisitor();
    	visitor.visitModel();
    	Program resultingProgram = visitor.getProgram();
    	
    	//make a results writer to write metrics measurements to file
    	ResultsWriter writer = new TextWriter(args[1]);
    	
    	//create metrics calculators to calculate various metrics
    	MemberMetricCalculator fieldMemberWriter = new MemberMetricCalculator(resultingProgram, new FieldMemberStrategy());
    	AccessMetricCalculator fieldAccessWriter = new AccessMetricCalculator(resultingProgram, new FieldAccessStrategy());
    	MemberMetricCalculator methodMemberWriter = new MemberMetricCalculator(resultingProgram, new MethodMemberStrategy());
    	AccessMetricCalculator methodAccessWriter = new AccessMetricCalculator(resultingProgram, new MethodAccessStrategy());

    	fieldMemberWriter.calculateResults(writer);
    	fieldAccessWriter.calculateResults(writer);
    	methodMemberWriter.calculateResults(writer);
    	methodAccessWriter.calculateResults(writer);
    }
}