package encapsulation.model;

import jst.symtab.*;
import java.util.*;

/**
 * Builder class, builds a simpler model of the program in memory for the metrics
 * calculators to extract information from
 * @author Janina Voigt
 *
 */
public class Builder {

	protected Program program;			//the program the builder is building up
	protected ClassOrInterface currentType;
	protected ExecutableBlock currentBlock;
	
	//accesses that cannot be resolved yet
	protected Map<ExecutableBlock, Map<Decl, List<String>>> accesses;
	
	/**
	 * Constructor for builder objects
	 */
	public Builder() {
		accesses = new HashMap<ExecutableBlock, Map<Decl, List<String>>>();
		program = new Program();
	}
	
	/**
	 * Build a user type (class or interface)
	 * @param theUserType the JST UserType corresponding to the user type being
	 * built
	 */
	public void addUserType(UserType theUserType){
		ClassOrInterface theType = new ClassOrInterface(theUserType);
		program.addClassOrInterface(theType);
		currentType = theType;
	}
	
	/**
	 * Build a field 
	 * @param theFieldDecl the JST FieldDecl corresponding to the field being
	 * built
	 */
	public void addField(FieldDecl theFieldDecl){
		Field theField = new Field(currentType, theFieldDecl);
		currentType.addField(theField);
	}
	
	/**
	 * Build a method 
	 * @param theMethodDecl the JST MethodDecl corresponding to the method being
	 * built
	 */
	public void addMethod(MethodDecl theMethodDecl){
		Method theMethod = new Method(currentType, theMethodDecl);
		currentType.addMethod(theMethod);
	}
	
	/**
	 * Build a block 
	 * @param executableCode the JST ExecutableCode corresponding to the block being
	 * built
	 */
	public void addBlock(ExecutableCode executableCode){
		ExecutableBlock block = new ExecutableBlock(currentType, executableCode);
		currentType.addBlock(block);
		currentBlock = block;
	}
	
	/**
	 * Add an access to a member
	 * @param accessedDecl the decl being accesses
	 * @param accessString the code string for the access
	 */
	public void addAccess(Decl accessedDecl, String accessString){
		Map<Decl, List<String>> accessedDecls = accesses.get(currentBlock);
		if(accessedDecls == null) {
			accessedDecls = new HashMap<Decl, List<String>>();
		}
		List<String> accessStrings = accessedDecls.get(accessedDecl);
		if(accessStrings == null){
			accessStrings = new ArrayList<String>();
		}
		accessStrings.add(accessString);
		accessedDecls.put(accessedDecl, accessStrings);
		//record the access for now and build it properly later
		accesses.put(currentBlock, accessedDecls);
	}
	
	/**
	 * Gets the program once the building process is finished
	 * @return the resulting program
	 */
	public Program getResult() {
		resolveAccesses();
		return program;
	}
	
	/**
	 * Resolves the accesses to members once the building process is finished
	 */
	private void resolveAccesses() {
		for(ExecutableBlock block : accesses.keySet()){
			Map<Decl, List<String>> accessedDecls = accesses.get(block);
			for(Decl decl : accessedDecls.keySet()){
				AccessibleMember member = program.getAccessibleMember(decl);
				if(member == null) {
					continue;
				}
				List<String> accessStrings = accessedDecls.get(decl);
				for(String accessString : accessStrings){
					Access access = new Access(member, block, accessString);
					member.addAccess(access);
					block.addAccess(access);
				}
			}
		}
	}
}
