package jst.parsetree;

import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import java.util.*;
import parser.parsetree.*;
import jst.app.*;
import parser.grammar.*;
import parser.parsetree.tokenFactory.*;
import parser.parsetree.visitor.PrintParseTreeVisitor;

/** This class reads a parse tree in XML format.
 *  and builds a Grammar object.
 */
public class XMLTreeLoader {

	protected DocumentBuilder domBuilder;

    protected TokenFactory tokenFactory;

	protected Terminal whitespaceTerminal = Symbol.grammar.getTerminal("WHITESPACE"); // Fix this.

    public XMLTreeLoader() {
    	// Create a factory for document builders.
        DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
        // domFactory.setValidating(true);

        tokenFactory = new SimpleTokenFactory();

		// Now use the factory to make the document builder.
        try {
        	domBuilder = domFactory.newDocumentBuilder();
        }
        catch(ParserConfigurationException e) {
			throw new RuntimeException(e); // Can't happen.
        }
	}

    public JstParseTreeBranch load(InputSource input, List messages) {

        Document document = null;

        try {
    		document = domBuilder.parse(input);
        }
        catch (IOException e) {
        	throw new RuntimeException(e); // Die disgracefully.
        }
        catch (SAXException e) {
            messages.add(new JSTMessage(JSTMessage.XML_PARSER_ERROR, e.getMessage()));
            return null;
        }

		NodeList kids = document.getChildNodes();
        if (kids == null || kids.getLength() != 1) {
            messages.add(new JSTMessage(JSTMessage.NOT_A_PARSETREE, input.getSystemId()));
            return null;
        }
        
		Node parseTreeNode = kids.item(0);
		if (parseTreeNode.getNodeType() != Node.ELEMENT_NODE || !parseTreeNode.getNodeName().equals("parsetree")) {
            messages.add(new JSTMessage(JSTMessage.NOT_A_PARSETREE, input.getSystemId()));
            return null;
        }

        kids = parseTreeNode.getChildNodes();
        Node goalNode = kids.item(1);
	    if (goalNode.getNodeType() != Node.ELEMENT_NODE || !goalNode.getNodeName().equals("goal")) {
            messages.add(new JSTMessage(JSTMessage.NOT_A_PARSETREE, input.getSystemId()));
            return null;
        }

        JstParseTreeBranch root = loadBranch(goalNode, messages);

		return messages.isEmpty()? root : null;
    }

	protected SyntaxTree loadNode(Node treeNode, List messages) {
        assert treeNode.getNodeType() == Node.ELEMENT_NODE;

        String nodeName = treeNode.getNodeName();

        if (nodeName == "token")
        	return loadLeaf(treeNode, messages);

        return loadBranch(treeNode, messages);
    }

    protected SyntaxTreeLeaf loadLeaf(Node terminalNode, List messages) {
        NamedNodeMap attributes = terminalNode.getAttributes();
	    String name = attributes.getNamedItem("id").getNodeValue();
	    Terminal terminal = Symbol.grammar.getTerminal(name);

        String lineString = attributes.getNamedItem("line").getNodeValue();
        int line = lineString == null? 0 : Integer.parseInt(lineString);

        String columnString = attributes.getNamedItem("column").getNodeValue();
        int column = columnString == null? 0 : Integer.parseInt(columnString);

        Node kid = terminalNode.getFirstChild();
        assert kid != null && kid.getNodeType() == Node.TEXT_NODE;

        String value = kid.getNodeValue();

        return tokenFactory.makeToken(terminal, value, line, column);
    }

    protected JstParseTreeBranch loadBranch(Node branchNode, List messages) {

		List resultKids = new ArrayList();
        NodeList kids = branchNode.getChildNodes();

        for (int i = 0; i < kids.getLength(); i++) {
        	Node kid = kids.item(i);
            if (kid.getNodeType() == Node.ELEMENT_NODE) {
			    SyntaxTree node = loadNode(kid, messages);
				if (node.getSymbolType() != whitespaceTerminal)
					resultKids.add(node);
			}
        }

        String ntName = branchNode.getNodeName();
        Production artificialProd = null;
        if (!ntName.equals("AMBIGUOUS")) {
            Nonterminal nt = Symbol.grammar.getNonterminal(ntName);
    
            List prods = nt.getProductions();
            if (prods.isEmpty())
                artificialProd = new Production(nt);
            else
                artificialProd = (Production) prods.get(0);
        }

        return new JstParseTreeBranch(artificialProd, resultKids);
    }
}
