package jst.symtab.visitor;
import jst.symtab.*;
import java.util.*;

public abstract class CompositionVisitor implements ModelVisitor {

    protected void visitKids(Collection kids) {
        Iterator iter = kids.iterator();
        while (iter.hasNext()) {
            Decl decl = (Decl) iter.next();

            decl.accept(this);
        }
    }

    public void visitDecl(Decl decl) {
    }

    public void visitPackageDecl(PackageDecl packageDecl) {
        visitDecl(packageDecl);

        visitKids(packageDecl.getSourceFiles());
        visitKids(packageDecl.getTypes());
        visitKids(packageDecl.getSubPackages());
    }

    public void visitSourceFile(SourceFile sourceFile) {
        visitDecl(sourceFile);
    }

    public void visitTypeDecl(TypeDecl typeDecl) {
        visitDecl(typeDecl);

        if (typeDecl.hasArrayType())
            typeDecl.getArrayType().accept(this);
    }

    public void visitPrimitiveType(PrimitiveType primitiveType) {
        visitTypeDecl(primitiveType);
    }

    public void visitNullType(NullType nullType) {
        visitTypeDecl(nullType);
    }

    public void visitActualType(ActualType actualType) {
        visitTypeDecl(actualType);
	}

    public void visitReferenceType(ReferenceType referenceType) {
        visitTypeDecl(referenceType);
    }

    public void visitArrayType(ArrayType arrayType) {
        visitReferenceType(arrayType);
    }

    public void visitUserType(UserType userType) {
        visitReferenceType(userType);

        visitKids(userType.getFields());
        visitKids(userType.getMethods());
        visitKids(userType.getTypes());
    }

    public void visitInterfaceType(InterfaceType interfaceType) {
        visitUserType(interfaceType);
    }

    public void visitGenericInterfaceType(GenericInterfaceType genericInterfaceType) {
		visitInterfaceType(genericInterfaceType);
	}

    public void visitParameterizedInterfaceType(ParameterizedInterfaceType parameterizedInterfaceType) {
		visitInterfaceType(parameterizedInterfaceType);
	}

    public void visitClassType(ClassType classType) {
        visitUserType(classType);

        visitKids(classType.getConstructors());
    }

    public void visitGenericClassType(GenericClassType genericClassType) {
		visitClassType(genericClassType);
	}

    public void visitParameterizedClassType(ParameterizedClassType parameterizedClassType) {
		visitClassType(parameterizedClassType);
	}

    public void visitTypedDecl(TypedDecl typedDecl) {
        visitDecl(typedDecl);
    }

    public void visitVariableDecl(VariableDecl variableDecl) {
        visitTypedDecl(variableDecl);
    }

    public void visitFieldDecl(FieldDecl fieldDecl) {
        visitVariableDecl(fieldDecl);
        
        if(fieldDecl != null && fieldDecl.getInitializer() != null){
        	fieldDecl.getInitializer().accept(this);
        }
    }

    public void visitLocalVariableDecl(LocalVariableDecl localVariableDecl) {
        visitVariableDecl(localVariableDecl);
    }

    public void visitParameterDecl(ParameterDecl parameterDecl) {
        visitVariableDecl(parameterDecl);
    }

    public void visitOperationDecl(OperationDecl operationDecl) {
        visitTypedDecl(operationDecl);

        visitKids(operationDecl.getParameters());
        
        Block body = operationDecl.getBody();
        if (body != null)
            body.accept(this);
    }

    public void visitMethodDecl(MethodDecl methodDecl) {
        visitOperationDecl(methodDecl);
    }

    public void visitConstructorDecl(ConstructorDecl constructorDecl) {
        visitOperationDecl(constructorDecl);
    }

    public void visitExecutableCode(ExecutableCode executableCode) {
        visitDecl(executableCode);
    }

    public void visitBlock(Block block) {
        visitExecutableCode(block);

        visitKids(block.getVariables());
        visitKids(block.getTypes());
        visitKids(block.getBlocks());
    }

    public void visitCatchBlock(CatchBlock catchBlock) {
        visitBlock(catchBlock);

        if(catchBlock.getParameter() != null) {
        	catchBlock.getParameter().accept(this);
        }
    }

    public void visitFieldInitializer(FieldInitializer fieldInitializer) {
        visitExecutableCode(fieldInitializer);

		visitKids(fieldInitializer.getAnonInnerClasses());
    }
    
    public void visitWildcardType(WildcardType wildcardType) {
    	visitActualType(wildcardType);

		if (wildcardType.getUpperBound() != null)
			wildcardType.getUpperBound().accept(this);

		if (wildcardType.getLowerBound() != null)
			wildcardType.getLowerBound().accept(this);
    }

    public void visitTypeParameter(TypeParameter typeParameter) {
    	visitActualType(typeParameter);

		typeParameter.getUpperBound().accept(this);
	}
}
