package parser.grammar;

import java.util.*;
//import yakyacc.pda.dynamicStateMachine.DottedItem;

/**
 * Production in a grammar.
 * <p>
 * For a particular grammar, you can define a subclass of Production
 * that contains lines of the form:
 *     public static final MyLanguageProductionType STMT_1 =
 *       new MyLanguageProductionType(
 *         MyLanguageNonterminalType.STMT,
 *         new Symbol[] {MyLanguageNonterminalType.DECL, MyLanguageTerminalType.SEMI});
 * for each production in the grammar.
 * <p>
 * Rhs may also be defined like this:
 *     STMT_1.addSymbol(MyLanguageNonterminalType.DECL);
 */
public class Production implements Comparable, ShiftOrReduce {

	/**
	 * Short name of this production, just for convenience.
	 */
	protected String name;

    /**
     * The nonterminal on the left-hand-side of this production.
     *
     * @supplierCardinality 1
     * @supplierRole lhs
     */
    protected Nonterminal lhs;

    /**
     * The list of Symbols on the right-hand-side .
     *
     * @associates parser.grammar.Symbol
     * @supplierCardinality 0..*
     * @supplierRole rhs
     */
    protected List rhs;  // Symbols.

	protected List items;

    // Special constructor used by FakeProduction subclass
    protected Production() {
    }

    /**
     * Construct a Production for the given lhs.
     * <p>
     * Call addSymbol() later to define the rhs.
     *
     * @param theLhs the nonterminal on the left.
     */
    public Production(Nonterminal theLhs) {
		this(null, theLhs);
	}

    /**
     * Construct a Production for the given lhs.
     * <p>
     * Call addSymbol() later to define the rhs.
     *
	 * @param String name identifies this production. A name is generated if null passed.
     * @param theLhs the nonterminal on the left.
     */
    public Production(String theName, Nonterminal theLhs) {
        assert theLhs != null;

		name = theName;
		if (name == null)
			name = theLhs.getName() + "_" + (theLhs.getProductions().size() + 1);

        setLhs(theLhs);
        rhs = new ArrayList();
        items = new ArrayList();
		items.add(new DottedItem(this, 0));
	}

    /**
     * Construct a Production for the given lhs and rhs.
     * Convenience constructor, primarily for code generators.
     *
     * @param theLhs the nonterminal on the left.
     * @param theRhs the symbols on the right.
     */
    public Production(Nonterminal theLhs, Symbol[] theRhs) {
        this(null, theLhs);
        assert theRhs != null;

        for (int i = 0; i <theRhs.length; i++)
            addSymbol(theRhs[i]);
	}

    /**
     * Construct a Production for the given lhs and rhs.
     * Convenience constructor, primarily for code generators.
     *
	 * @param String name identifies this production. A name is generated if null passed.
     * @param theLhs the nonterminal on the left.
     * @param theRhs the symbols on the right.
     */
    public Production(String theName, Nonterminal theLhs, Symbol[] theRhs) {
        this(theName, theLhs);
        assert theRhs != null;

        for (int i = 0; i <theRhs.length; i++)
            addSymbol(theRhs[i]);
	}

    public int compareTo(Object o) {
        Production that = (Production) o;

        return getName().compareTo(that.getName());
    }

	/**
	 * Getter for name.
	 */
	public String getName() {
		return name;
	}

    /**
     * Setter for lhs.  Builds reciprocal relationship
     * @param theLhs the nonterminal on the left.
     */
    protected void setLhs(Nonterminal theLhs) {
        lhs = theLhs;
        lhs.addProduction(this); // Reciprocal relationship.
    }

    /** Getter for lhs. */
    public Nonterminal getLhs() {
        return lhs;
    }

    /**
     * Extend the rhs with another symbol.
     * @param symbol the symbol to be appended to the rhs.
     */
    public void addSymbol(Symbol symbol) {
        assert symbol != null;

        rhs.add(symbol);
		items.add(new DottedItem(this, rhs.size()));
    }

    /**
     * Getter for the rhs.
     * @return the rhs list; do not modify.
     */
    public List getRhs() {
        return rhs;
    }

    /** Getter for size of rhs.
     *  @return the number of symbols on the rhs.
     */
    public int getSize() {
        return rhs.size();
    }

    public boolean isShift() {
        return false;
    }

    public Terminal asShift() {
        return null;
    }

    public Production asReduce() {
        return this;
    }

	public List getItems() {
		return items;
	}

    public String toString() {
        String result = getLhs().toString() + " ::=";
        for (int i = 0; i < rhs.size(); i++) {
            Symbol s = (Symbol) rhs.get(i);
	        result += " " + s.toString();
        }
        return result;
    }
}
