package parser.parsetree;

import parser.grammar.*;
import parser.parsetree.visitor.*;
import java.util.Set;

/** Abstract superclass for a lexeme found in a source file. */
public class Token implements SyntaxTreeLeaf {

    /**
     * The terminal symbol from the grammar that defines the type of this token.
     * 
     * @supplierCardinality 1
     * @clientCardinality 0..* 
     */
	protected Terminal terminalType;

    /** Line on which the token starts. */
	protected short line;

    /** Column on which the token starts. */
	protected short column;

	protected SyntaxTreeBranch parent;
    /**
     * Next token in the chain.
     * 
     * @supplierCardinality 0..1
     * @supplierRole next 
     */
	protected SyntaxTreeLeaf next;

    /** Construct a Token.
     *  @param theTerminalType The type of the token represented by this leaf node.
     */
    public Token(Terminal theTerminalType, int theLine, int theColumn) {
        assert theTerminalType != null;
        assert theLine <= Short.MAX_VALUE : "line is" + theLine;
	    assert theColumn <= Short.MAX_VALUE : "column is " + theColumn;

        terminalType = theTerminalType;
        line = (short) theLine;
	    column = (short) theColumn;
    }

    /**
     * Accept a visitor.  GoF Visitor pattern variant.
     *
     * @param visitor the visitor.
     */
    public void accept(SyntaxTreeVisitor visitor) {
	    visitor.visitLeaf(this);
    }

    /** Getter for terminalType. */
    public Symbol getSymbolType() {
        return getTerminalType();
    }

    /** Getter for terminalType. */
    public Terminal getTerminalType() {
        return terminalType;
    }

    /** Getter for line. */
	public int getLine() {
	    return line;
	}

    /** Getter or column. */
	public int getColumn() {
	    return column;
	}

    public boolean hasValue() {
        return false;
    }

    /** Get the string value of the token.  Uses name of terminal symbol. */
        public String getValue() {
	    return terminalType.toString();
    }
        
	public void setParent(SyntaxTreeBranch theParent) {
		parent = theParent;
	}
	
	public SyntaxTreeBranch getParent() {
		return parent;
	}

    /** Set the successor to this token. */
	public void setNext(SyntaxTreeLeaf theNext) {
	    next = theNext;
	}

    /** Getter for next. */
    public SyntaxTreeLeaf getNext() {
        return next;
    }

    public SyntaxTreeBranch asBranch() {
        return null;
    }

    public SyntaxTreeLeaf asLeaf() {
        return this;
    }

	public String toString() {
        return terminalType + " (" + getLine() + ", " + getColumn() + ")";
    }

}
