package parser.pda.stateMachine.builder;

import parser.parsetree.*;
import parser.grammar.*;
import parser.pda.stateMachine.*;

import java.util.*;

public class GenStateMachineBuilder {
    /**
     * @supplierRole machineBuilder
     * @supplierCardinality 1 
     */
    protected StateMachineBuilder machineBuilder;

    protected Map productions; // String ==> Production
    protected Grammar grammar;

    public GenStateMachineBuilder(LookaheadBuilder lookBuilder) {
        machineBuilder = new StateMachineBuilder(lookBuilder);
        productions = new HashMap();
        grammar = new Grammar();
    }

    public void makeTerminals(String[] terminalArray) {
        for (int i = 0; i < terminalArray.length; i++)
            grammar.getTerminal(terminalArray[i]);
    }

    public void makeNonterminals(String[] nonterminalArray) {
        for (int i = 0; i < nonterminalArray.length; i++)
            grammar.getNonterminal(nonterminalArray[i]);
    }

    // X_1 X A B C ==> name, lhs, rhs1 rhs2 rhs3
    public void makeProductions(String[][] productionArray) {

        for (int i = 0; i < productionArray.length; i++) {
            String prodName = productionArray[i][0];
            String lhsName = productionArray[i][1];

            Nonterminal lhs = grammar.getNonterminal(lhsName);
            Symbol[] rhs = new Symbol[productionArray[i].length - 2];

            for (int j = 2; j < productionArray[i].length; j++) {
                String symbolName = productionArray[i][j];
                rhs[j-2] = grammar.getSymbol(symbolName);
            }

            Production prod = new Production(lhs, rhs);
            productions.put(prodName, prod);
        }
    }

    // X_1_1 a ==> statename symbol
    public void makeStates(String[][] stateArray) {
        for (int i = 0; i < stateArray.length; i++) {
            String stateName = stateArray[i][0];

            String symbolName = stateArray[i][1];
            Symbol symbol = grammar.getSymbol(symbolName);

            machineBuilder.state(stateName, symbol);
        }
    }

    // Default way of building a PDA: make all states, make all actions, make all lookaheads.
    // Unfortunately, this requires arrays too large for static initialisation in Java,
    // so we use an alternative initState() approach which seperately defines each state.

    // X_1_1 X_1_2 Y_1_1 X_1 ==> statename dst1 dst2 red1
    public void makeActions(String[][] actionArray) {
        for (int i = 0; i < actionArray.length; i++) {
            String stateName = actionArray[i][0];
            machineBuilder.beginState(stateName);

            for (int j = 1; j < actionArray[i].length; j++) {
                String id = actionArray[i][j];

                Production prod = (Production) productions.get(id);
                if (prod == null)
                    machineBuilder.transition(id);
                else
                    machineBuilder.reduction(prod);
            }

            machineBuilder.endState();
        }
    }

    // X_1_1 a b c !S_a !R_X_1 ==> stateName look1 look2 look3 shift1 reduce1
    public void makeLookahead(String[][] lookaheadArray) {
        String prevState = null;

        for (int i = 0; i < lookaheadArray.length; i++) {
            String stateName = lookaheadArray[i][0];

            if (stateName != prevState) { // Starting a new state.
                if (prevState != null)
                    machineBuilder.endState();

                machineBuilder.beginState(stateName);
                prevState = stateName;
            }

            int depth = 0;
            for (int j = 1; j < lookaheadArray[i].length; j++) {
                String s = lookaheadArray[i][j];
                if (s.startsWith("!S_")) {
                    Terminal shiftTerminal = grammar.getTerminal(s.substring(3));
                    machineBuilder.addAction(shiftTerminal);
                }
                else if (s.startsWith("!R_")) {
                    Production reduceProd = (Production) productions.get(s.substring(3));
                    machineBuilder.addAction(reduceProd);
                }
                else {
                    Terminal look = grammar.getTerminal(s);
                    machineBuilder.beginOn(look);
                    depth++;
                }
            }

            while (depth-- > 0)
                machineBuilder.endOn();
        }

        if (prevState != null)
            machineBuilder.endState();
    }

    // items: production-name, dotpos
    // transitions: dst-state-name
    // reductions: production-names
    // lookahead: a b c !S_a !R_prod !on !on !on
    public void initState(String stateName, String[][] items, String[] transitions, String[] reductions, String[] lookahead) {
        machineBuilder.beginState(stateName);

		for (int i = 0; i < items.length; i++) {
			String prodName = items[i][0];
			int dotPos = Integer.parseInt(items[i][1]);

			Production prod = (Production) productions.get(prodName);
			DottedItem item = (DottedItem) prod.getItems().get(dotPos);

			machineBuilder.addItem(item);
		}

        for (int t = 0; t < transitions.length; t++)
            machineBuilder.transition(transitions[t]);

        for (int r = 0; r < reductions.length; r++)
            machineBuilder.reduction((Production) productions.get(reductions[r]));

        for (int l = 0; l < lookahead.length; l++) {
            String s = lookahead[l];
            if (s.startsWith("!S_")) {
                Terminal shiftTerminal = grammar.getTerminal(s.substring(3));
                machineBuilder.addAction(shiftTerminal);
            }
            else if (s.startsWith("!R_")) {
                Production reduceProd = (Production) productions.get(s.substring(3));
                machineBuilder.addAction(reduceProd);
            }
            else if (s.startsWith("!on")) {
                machineBuilder.endOn();
            }
            else {
                Terminal look = grammar.getTerminal(s);
                machineBuilder.beginOn(look);
            }
        }

        machineBuilder.endState();
    }

    /*
    public Map getStates() {
        return machineBuilder.getStates();
    }
    */

    public void makeMachine(String startName, String stopName) {
        machineBuilder.makeMachine(startName, stopName);
    }

    public StateMachine getResult() {
        return machineBuilder.getResult();
    }

    // Just for testing
    public Grammar getGrammar() {
        return grammar;
    }
}
